import argparse as ap

from redirector.utils import generators
from redirector.utils import logger
from redirector.utils import parser


class Redirector:
    def __init__(self, logger=None):
        self.parser = parser.ConfigReader(logger=logger)
        self.generator = generators.Generator()

    def generate(self, yaml_file, map_file):
        project_name = "Error"
        try:
            data, project_name = self.parser.parse_map(map_file, yaml_file)
            if not project_name:
                raise self.RedirectorInputDataError("Can\'t properly parse .map or .yaml files", Exception())
        # FIXME: what is the better way to do so?
        except Exception as e:
            raise self.RedirectorParserError("Can\'t parse .map file %s" % map_file, e)

        self.generator.generate(data, project_name)

    class RedirectorParserError(Exception):
        def __init__(self, message, errors):
            super().__init__(message)
            self.errors = errors

    class RedirectorInputDataError(Exception):
        def __init__(self, message, errors):
            super().__init__(message)
            self.errors = errors


def main(args=None):
    parser = ap.ArgumentParser(description='Redirector: nginx config generator')
    parser.add_argument('yaml_file', metavar='Y', type=str, nargs=1, help='.yaml file that defines project and it\'s content')
    parser.add_argument('map_file', metavar='M', type=str, nargs=1, help='.map file that defines redirects for project')
    if not args:
        args = parser.parse_args()

    redirector = Redirector()
    try:
        redirector.generate(args.yaml_file[0], args.map_file[0])
    except (generators.Generator.GenerationError, redirector.RedirectorParserError) as e:
        print("CRITICAL:\n" + str(e))


if __name__ == "__main__":
    main()

